@@ -3,22 +3,22 @@ package Convert::Pheno::PXF;
 use strict;
 use warnings;
 use autodie;
-use feature                 qw(say);
-use Convert::Pheno::Default qw(get_defaults);
-use Convert::Pheno::Mapping;
+use feature qw(say);
+use Data::Dumper;
+use Convert::Pheno::Utils::Default qw(get_defaults);
+use Convert::Pheno::Utils::Mapping;
 use Exporter 'import';
 our @EXPORT = qw(do_pxf2bff);
 
+my $DEFAULT = get_defaults();
+
 #############
 #############
 #  PXF2BFF  #
 #############
 #############
 
-my $DEFAULT = get_defaults();
-
 sub do_pxf2bff {
-
     my ( $self, $data ) = @_;
     my $sth = $self->{sth};
 
@@ -80,37 +80,110 @@ sub do_pxf2bff {
     # NB: In PXF some terms are = []
 
     # Initiate BFF structure
-    my $individual;
+    my $individual = {};
 
     # ========
     # diseases
     # ========
+    _map_diseases( $phenopacket, $individual );
 
+    # ========
+    # exposures
+    # ========
+    _map_exposures( $phenopacket, $individual );
+
+    # ================
+    # geographicOrigin
+    # ================
+    # NA
+
+    # ==
+    # id
+    # ==
+    _map_id( $phenopacket, $individual );
+
+    # ====
+    # info
+    # ====
+    _map_info( $phenopacket, $cohort, $family, $individual );
+
+    # =========================
+    # interventionsOrProcedures
+    # =========================
+    _map_interventions_or_procedures( $phenopacket, $individual );
+
+    # =============
+    # karyotypicSex
+    # =============
+    _map_karyotypicSex( $phenopacket, $individual );
+
+    # =========
+    # measures
+    # =========
+    _map_measures( $phenopacket, $individual );
+
+    # =========
+    # pedigrees
+    # =========
+    # See above {info}{phenopacket}{pedigree} => singular!!!
+
+    # ==================
+    # phenotypicFeatures
+    # ==================
+    _map_phenotypic_features( $phenopacket, $individual );
+
+    # ===
+    # sex
+    # ===
+    _map_sex( $self, $phenopacket, $individual );
+
+    # ==========
+    # treatments
+    # ==========
+    _map_treatments( $phenopacket, $individual );
+
+    ##################################
+    # END MAPPING TO BEACON V2 TERMS #
+    ##################################
+
+    # print Dumper $individual;
+    return $individual;
+}
+
+################################################################################
+# Helper subs extracted from do_pxf2bff (preserving original comments)
+################################################################################
+
+sub _map_diseases {
+    my ( $phenopacket, $individual ) = @_;
+
+    # ========
+    # diseases
+    # ========
     if ( exists $phenopacket->{diseases} ) {
         for my $pxf_disease ( @{ $phenopacket->{diseases} } ) {
             my $disease = $pxf_disease;    # Ref-copy-only
             $disease->{diseaseCode} = $disease->{term};
             $disease->{ageOfOnset}  = $disease->{onset}
               if exists $disease->{onset};
-            $disease->{excluded} =
-              ( exists $disease->{negated} || exists $disease->{excluded} )
-              ? JSON::XS::true
-              : JSON::XS::false;
+
+            # Check and normalize keys if they exist
+            for (qw/excluded negated/) {
+                $disease->{$_} = $disease->{$_} if exists $disease->{$_};
+            }
 
             # Clean analog terms if exist
             for (qw/term onset/) {
-                delete $disease->{$_}
-                  if exists $disease->{$_};
+                delete $disease->{$_} if exists $disease->{$_};
             }
 
             push @{ $individual->{diseases} }, $disease;
         }
     }
+}
 
-    # ========
-    # ethnicity
-    # ========
-    # NA
+sub _map_exposures {
+    my ( $phenopacket, $individual ) = @_;
 
     # ========
     # exposures
@@ -131,36 +204,37 @@ sub do_pxf2bff {
 
             # Clean analog terms if exist
             for (qw/type occurence/) {
-                delete $exposure->{$_}
-                  if exists $exposure->{$_};
+                delete $exposure->{$_} if exists $exposure->{$_};
             }
 
             push @{ $individual->{exposures} }, $exposure;
         }
     }
+}
 
-    # ================
-    # geographicOrigin
-    # ================
-    # NA
+sub _map_id {
+    my ( $phenopacket, $individual ) = @_;
 
     # ==
     # id
     # ==
+    if ( exists $phenopacket->{subject}{id} ) {
+        $individual->{id} = $phenopacket->{subject}{id};
+    }
+}
 
-    $individual->{id} = $phenopacket->{subject}{id}
-      if exists $phenopacket->{subject}{id};
+sub _map_info {
+    my ( $phenopacket, $cohort, $family, $individual ) = @_;
 
     # ====
     # info
     # ====
-
     # *** IMPORTANT ***
     # Here we set data that do not fit anywhere else
 
     # Miscelanea for top-level 'phenopacket'
     for my $term (
-        qw (dateOfBirth genes interpretations metaData variants files biosamples pedigree)
+        qw(dateOfBirth genes interpretations metaData variants files biosamples pedigree)
       )
     {
         $individual->{info}{phenopacket}{$term} = $phenopacket->{$term}
@@ -170,11 +244,14 @@ sub do_pxf2bff {
     # Miscelanea for top-levels 'cohort' and 'family'
     $individual->{info}{cohort} = $cohort if defined $cohort;
     $individual->{info}{family} = $family if defined $family;
+}
+
+sub _map_interventions_or_procedures {
+    my ( $phenopacket, $individual ) = @_;
 
     # =========================
     # interventionsOrProcedures
-    # ========================
-
+    # =========================
     if ( exists $phenopacket->{medicalActions} ) {
         for my $action ( @{ $phenopacket->{medicalActions} } ) {
             if ( exists $action->{procedure} ) {
@@ -190,21 +267,28 @@ sub do_pxf2bff {
 
                 # Clean analog terms if exist
                 for (qw/code performed/) {
-
-                    delete $procedure->{$_}
-                      if exists $procedure->{$_};
+                    delete $procedure->{$_} if exists $procedure->{$_};
                 }
 
                 push @{ $individual->{interventionsOrProcedures} }, $procedure;
             }
         }
     }
+}
+
+sub _map_karyotypicSex {
+    my ( $phenopacket, $individual ) = @_;
 
     # =============
     # karyotypicSex
     # =============
-    $individual->{karyotypicSex} = $phenopacket->{subject}{karyotypicSex}
-      if exists $phenopacket->{subject}{karyotypicSex};
+    if ( exists $phenopacket->{subject}{karyotypicSex} ) {
+        $individual->{karyotypicSex} = $phenopacket->{subject}{karyotypicSex};
+    }
+}
+
+sub _map_measures {
+    my ( $phenopacket, $individual ) = @_;
 
     # =========
     # measures
@@ -215,12 +299,12 @@ sub do_pxf2bff {
 
             $measure->{assayCode} = $measure->{assay};
 
-            # Process remotely compleValue
+            # Process remotely complexValue
             # s/type/quantityType/
             map_complexValue( $measure->{complexValue} )
               if exists $measure->{complexValue};
 
-            # Assign dependeing on PXF
+            # Assign depending on PXF
             $measure->{measurementValue} =
                 exists $measure->{value}        ? $measure->{value}
               : exists $measure->{complexValue} ? $measure->{complexValue}
@@ -230,18 +314,16 @@ sub do_pxf2bff {
 
             # Clean analog terms if exist
             for (qw/assay value complexValue/) {
-                delete $measure->{$_}
-                  if exists $measure->{$_};
+                delete $measure->{$_} if exists $measure->{$_};
             }
 
             push @{ $individual->{measures} }, $measure;
         }
     }
+}
 
-    # =========
-    # pedigrees
-    # =========
-    # See above {info}{phenopacket}{pedigree} => singular!!!
+sub _map_phenotypic_features {
+    my ( $phenopacket, $individual ) = @_;
 
     # ==================
     # phenotypicFeatures
@@ -253,12 +335,13 @@ sub do_pxf2bff {
             # *** IMPORTANT ****
             # In v2.0.0 BFF 'evidence' is object but in PXF is array of objects
 
-            $phenotypicFeature->{excluded} =
-              (      exists $phenotypicFeature->{negated}
-                  || exists $phenotypicFeature->{excluded} )
-              ? JSON::XS::true
-              : JSON::XS::false,
-              $phenotypicFeature->{featureType} = $phenotypicFeature->{type}
+            # Check and normalize keys if they exist
+            for (qw/excluded negated/) {
+                $phenotypicFeature->{excluded} = $phenotypicFeature->{$_}
+                  if exists $phenotypicFeature->{$_};
+            }
+
+            $phenotypicFeature->{featureType} = $phenotypicFeature->{type}
               if exists $phenotypicFeature->{type};
 
             # Clean analog terms if exist
@@ -270,26 +353,34 @@ sub do_pxf2bff {
             push @{ $individual->{phenotypicFeatures} }, $phenotypicFeature;
         }
     }
+}
+
+sub _map_sex {
+    my ( $self, $phenopacket, $individual ) = @_;
 
     # ===
     # sex
     # ===
+    if ( exists $phenopacket->{subject}{sex}
+        && $phenopacket->{subject}{sex} ne '' )
+    {
+        $individual->{sex} = map_ontology_term(
+            {
+                query    => $phenopacket->{subject}{sex},
+                column   => 'label',
+                ontology => 'ncit',
+                self     => $self
+            }
+        );
+    }
+}
 
-    $individual->{sex} = map_ontology_term(
-        {
-            query    => $phenopacket->{subject}{sex},
-            column   => 'label',
-            ontology => 'ncit',
-            self     => $self
-        }
-      )
-      if ( exists $phenopacket->{subject}{sex}
-        && $phenopacket->{subject}{sex} ne '' );
+sub _map_treatments {
+    my ( $phenopacket, $individual ) = @_;
 
     # ==========
     # treatments
     # ==========
-
     if ( exists $phenopacket->{medicalActions} ) {
         for my $action ( @{ $phenopacket->{medicalActions} } ) {
             if ( exists $action->{treatment} ) {
@@ -300,8 +391,7 @@ sub do_pxf2bff {
                   : $DEFAULT->{ontology_term};
 
                 # Clean analog terms if exist
-                delete $treatment->{agent}
-                  if exists $treatment->{agent};
+                delete $treatment->{agent} if exists $treatment->{agent};
 
                 # doseIntervals needs some parsing
                 if ( exists $treatment->{doseIntervals} ) {
@@ -309,7 +399,6 @@ sub do_pxf2bff {
                     # Required properties:
                     #   - scheduleFrequency
                     #   - quantity
-
                     for ( @{ $treatment->{doseIntervals} } ) {
 
                         # quantity
@@ -317,7 +406,7 @@ sub do_pxf2bff {
                             $_->{quantity} = $DEFAULT->{quantity};
                         }
 
-                        #scheduleFrequency
+                        # scheduleFrequency
                         unless ( exists $_->{scheduleFrequency} ) {
                             $_->{scheduleFrequency} = $DEFAULT->{ontology_term};
                         }
@@ -328,17 +417,13 @@ sub do_pxf2bff {
             }
         }
     }
-
-    ##################################
-    # END MAPPING TO BEACON V2 TERMS #
-    ##################################
-
-    # print Dumper $individual;
-    return $individual;
 }
 
-sub map_complexValue {
+#----------------------------------------------------------------------
+# Helper subs
+#----------------------------------------------------------------------
 
+sub map_complexValue {
     my $complexValue = shift;
 
     # "typedQuantities": [
@@ -357,7 +442,7 @@ sub map_complexValue {
     #            }
     #  }
 
-    # Modifying the orginal ref
+    # Modifying the original ref
     for ( @{ $complexValue->{typedQuantities} } ) {
         $_->{quantityType} = delete $_->{type};
     }
@@ -365,4 +450,13 @@ sub map_complexValue {
     return 1;
 }
 
+# Function to normalize a value to a Boolean
+sub to_boolean {
+    my $value = shift;
+    print Dumper $value;
+    return JSON::XS::true
+      if $value && $value ne 'false';    # Non-empty string and not 'false'
+    return JSON::XS::false;              # Empty, 'false', or undef
+}
+
 1;
